<?php

namespace Drupal\crossword_image;

use Drupal\file\FileInterface;

/**
 * Interface for plugins that create images from crossword files.
 */
interface CrosswordImagePluginInterface {

  /**
   * Returns the uri to an image representation of the crossword file.
   *
   * If the image does not exist yet, it will be created and saved.
   *
   * @param Drupal\file\FileInterface $file
   *   The file representing the crossword puzzle.
   *
   * @return string
   *   The uri to an image version of the crossword.
   */
  public function getImageUri(FileInterface $file);

  /**
   * Returns an image resource representing a preview of the puzzle.
   *
   * @param array $data
   *   The parsed crossword file. See CrosswordFileParserPluginBase.
   *
   * @return resource
   *   An image resource to be used as a thumbnail.
   */
  public function createImageResource(array $data);

}
