<?php

namespace Drupal\crossword_image\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\crossword\Plugin\Field\FieldFormatter\CrosswordFileDownloadLink;

/**
 * Plugin implementation of the 'crossword_image_download' formatter.
 *
 * @FieldFormatter(
 *   id = "crossword_image_download",
 *   label = @Translation("Crossword Image (download)"),
 *   field_types = {
 *     "crossword"
 *   },
 * )
 */
class CrosswordImageDownload extends CrosswordFileDownloadLink {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return ['crossword_image' => 'thumbnail'] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $form['crossword_image'] = [
      '#type' => 'select',
      '#options' => \Drupal::service('crossword.manager.image')->getCrosswordImageOptionList(),
      '#default_value' => $this->getSetting('crossword_image'),
      '#title' => $this->t('Crossword Image Type'),
      '#description' => $this->t('Choose the Crossword Image Plugin to be used when generating an image from the Crossword file.'),
      '#weight' => -100,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $image_type = $this->t('Image Type: %crossword_image', ['%crossword_image' => $this->getSetting('crossword_image')]);
    array_unshift($summary, $image_type);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {
      $image_uri = \Drupal::service('crossword.manager.image')->createInstance($this->getSetting('crossword_image'))->getImageUri($file);
      $elements[$delta]['#url'] = Url::fromUri(file_create_url($image_uri));
    }

    return $elements;
  }

}
