<?php

namespace Drupal\crossword_image\Plugin\crossword\crossword_image;

/**
 * This makes a image of the solved puzzle.
 *
 * @CrosswordImage(
 *   id = "solution_thumbnail",
 *   title = @Translation("Solution Thumbnail"),
 *   type = IMAGETYPE_PNG,
 *   toolkit = "gd"
 * )
 */
class CrosswordSolutionThumbnail extends CrosswordThumbnailBase {

  /**
   * {@inheritdoc}
   */
  protected $squareSize = 40;

  /**
   * {@inheritdoc}
   */
  protected $lineSize = 3;

  /**
   * {@inheritdoc}
   */
  public function createImageResource(array $data) {
    $image = parent::createImageResource($data);
    $grid = $data['puzzle']['grid'];
    $square_size = $this->squareSize;
    $black = imagecolorallocate($image, 0, 0, 0);
    $font = drupal_get_path('module', 'crossword_image') . '/fonts/RobotoMono-Regular.ttf';
    // The RobotoMono regular has approximately this aspect ratio.
    $font_aspect = 1.29;
    foreach ($grid as $row_index => $row) {
      foreach ($row as $col_index => $square) {
        if (isset($square['fill'])) {
          if (!empty($square['rebus'])) {
            $angle = -45;
            // Prescribe a total width for the rebus text.
            $text_width = $square_size / sqrt(2);
            $size = $text_width / strlen($square['fill']) * $font_aspect;
            // Bottom left of text. Kind of magic.
            $x = ($col_index + 0.15) * $square_size;
            $y = ($row_index + 0.41) * $square_size - sqrt($size) / 4;
          }
          else {
            $size = $this->squareSize / 2;
            $angle = 0;
            // Bottom left of text.
            $x = ($col_index + 0.5) * $square_size - $size / 2 / $font_aspect;
            $y = ($row_index + 1) * $square_size - $size / 2;
          }
          imagettftext($image, $size, $angle, $x, $y, $black, $font, $square['fill']);
        }
        if (!empty($square['circle'])) {
          $start = -180;
          $end = 180;
          $height = $width = $this->squareSize;
          $x = ($col_index + 0.5) * $square_size;
          $y = ($row_index + 0.5) * $square_size;
          // Draw two arcs to make the line thicker.
          imagearc($image, $x, $y, $width, $height, $start, $end, $black);
          imagearc($image, $x, $y, $width - 1, $height - 1, $start, $end, $black);
        }
      }
    }
    return $image;
  }

}
