<?php

namespace Drupal\crossword_image\Plugin\crossword\crossword_image;

use Drupal\crossword_image\CrosswordImagePluginBase;

/**
 * A base class for thumbnail images of crossword files.
 *
 * Simple extensions of this can set the image type, square size,
 * and line width.
 */
abstract class CrosswordThumbnailBase extends CrosswordImagePluginBase {

  /**
   * The width of a square in pixels.
   *
   * @var int
   */
  protected $squareSize;

  /**
   * The width of a line in pixels.
   *
   * @var int
   */
  protected $lineSize;

  /**
   * {@inheritdoc}
   */
  public function createImageResource(array $data) {
    $grid = $data['puzzle']['grid'];
    $square_size = $this->squareSize;
    $line_size = $this->lineSize;
    $width = count($grid[0]) * $square_size + $line_size / 2;
    $height = count($grid) * $square_size + $line_size / 2;
    $image = imagecreatetruecolor($width, $height);
    $white = imagecolorallocate($image, 255, 255, 255);
    foreach ($grid as $row_index => $row) {
      foreach ($row as $col_index => $square) {
        if ($square['fill'] !== NULL) {
          $color = $white;
          imagefilledrectangle($image, $col_index * $square_size + $line_size / 2, $row_index * $square_size + $line_size / 2, ($col_index + 1) * $square_size - $line_size / 2, ($row_index + 1) * $square_size - $line_size / 2, $color);
        }
      }
    }
    return $image;
  }

}
