<?php

/**
 * @file
 * Token callbacks for the crossword module.
 */

use Drupal\Core\Render\BubbleableMetadata;

/**
 * Implements hook_token_info().
 */
function crossword_token_token_info() {
  $info['tokens']['file']['crossword_author'] = [
    'name' => t('Crossword Author'),
    'description' => t('Author of the Crossword'),
  ];
  $info['tokens']['file']['crossword_title'] = [
    'name' => t('Crossword Title'),
    'description' => t('Title of the Crossword'),
  ];
  $info['tokens']['file']['crossword_dimensions'] = [
    'name' => t('Crossword Dimensions'),
    'description' => t('Dimensions of the Crossword'),
  ];
  $info['tokens']['file']['crossword_dimension_across'] = [
    'name' => t('Crossword Dimension Across'),
    'description' => t('Dimension of the Crossword'),
  ];
  $info['tokens']['file']['crossword_dimension_down'] = [
    'name' => t('Crossword Dimension Down'),
    'description' => t('Dimension of the Crossword'),
  ];

  return $info;
}

/**
 * Implements hook_tokens().
 */
function crossword_token_tokens($type, $tokens, array $data = [], array $options = [], BubbleableMetadata $bubbleable_metadata) {
  $replacements = [];
  // Crossword tokens, which are special file tokens.
  if ($type == 'file' && !empty($data['file'])) {
    $file = $data['file'];

    foreach ($tokens as $name => $original) {
      switch ($name) {
        case 'crossword_author':
          $data_service = \Drupal::service('crossword.data_service');
          $author = $data_service->getAuthor($file);
          if ($author) {
            $replacements[$original] = $author;
          }
          break;

        case 'crossword_title':
          $data_service = \Drupal::service('crossword.data_service');
          $title = $data_service->getTitle($file);
          if ($title) {
            $replacements[$original] = $title;
          }
          break;

        case 'crossword_dimensions':
          $data_service = \Drupal::service('crossword.data_service');
          $dimensions = $data_service->getDimensions($file);
          if ($dimensions) {
            $replacements[$original] = $dimensions;
          }
          break;

        case 'crossword_dimension_down':
          $data_service = \Drupal::service('crossword.data_service');
          $dimension = $data_service->getDimensionDown($file);
          if ($dimension) {
            $replacements[$original] = $dimension;
          }
          break;

        case 'crossword_dimension_across':
          $data_service = \Drupal::service('crossword.data_service');
          $dimension = $data_service->getDimensionAcross($file);
          if ($dimension) {
            $replacements[$original] = $dimension;
          }
          break;
      }
    }
  }
  return $replacements;
}
