<?php

namespace Drupal\Tests\crossword\Kernel;

use Drupal\KernelTests\KernelTestBase;

/**
 * Covers CrosswordDataService.
 *
 * @group crossword
 */
class CrosswordDataServiceTest extends KernelTestBase {

  /**
   * The modules to load to run the test.
   *
   * @var array
   */
  public static $modules = ['system', 'crossword', 'file', 'user'];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->installConfig(['file', 'user']);
    $this->installSchema('user', ['users_data']);
    $this->installSchema('file', ['file_usage']);
    $this->installEntitySchema('user');
    $this->installEntitySchema('file');
  }

  /**
   * Test data service against test puzzle.
   */
  public function testCrosswordDataService() {
    $contents = file_get_contents(drupal_get_path('module', 'crossword') . "/tests/files/test.txt");
    $file = file_save_data($contents, "public://test.txt");
    $expected_json = file_get_contents(drupal_get_path('module', 'crossword') . "/tests/files/test.json");
    $expected_data = json_decode($expected_json, TRUE);
    $expected_data['id'] = $file->id();

    $crossword_data_service = \Drupal::service('crossword.data_service');
    $data = $crossword_data_service->getData($file);
    $this->assertTrue($data == $expected_data);

    $title = $crossword_data_service->getTitle($file);
    $this->assertEquals('Test Puzzle', $title);

    $author = $crossword_data_service->getAuthor($file);
    $this->assertEquals('Test Author', $author);

    $dimensions = $crossword_data_service->getDimensions($file, ' by ');
    $this->assertEquals('3 by 3', $dimensions);

    $dimension_across = $crossword_data_service->getDimensionAcross($file);
    $this->assertEquals(3, $dimension_across);

    $dimension_down = $crossword_data_service->getDimensionDown($file);
    $this->assertEquals(3, $dimension_down);

    $is_rebus = $crossword_data_service->isRebus($file);
    $this->assertTrue($is_rebus);
  }

}
